@@ -1,3 +1,4 @@
+use crate::util::path_eq;
 use crate::Diagnostic;
 use proc_macro2::TokenStream as TokenStream2;
 use quote::quote;
@@ -15,14 +16,11 @@ enum ParameterKind {
 
 impl ParameterKind {
     fn from_ident(ident: &Ident) -> Option<ParameterKind> {
-        if ident == "positional_only" {
-            Some(ParameterKind::PositionalOnly)
-        } else if ident == "positional_or_keyword" {
-            Some(ParameterKind::PositionalOrKeyword)
-        } else if ident == "keyword_only" {
-            Some(ParameterKind::KeywordOnly)
-        } else {
-            None
+        match ident.to_string().as_str() {
+            "positional_only" => Some(ParameterKind::PositionalOnly),
+            "positional_or_keyword" => Some(ParameterKind::PositionalOrKeyword),
+            "keyword_only" => Some(ParameterKind::KeywordOnly),
+            _ => None,
         }
     }
 }
@@ -45,7 +43,9 @@ impl ArgAttribute {
                     err_span!(list, "There must be at least one argument to #[pyarg()]")
                 })?;
                 let kind = match first_arg {
-                    NestedMeta::Meta(Meta::Word(ident)) => ParameterKind::from_ident(ident),
+                    NestedMeta::Meta(Meta::Path(path)) => {
+                        path.get_ident().and_then(ParameterKind::from_ident)
+                    }
                     _ => None,
                 };
                 let kind = kind.ok_or_else(|| {
@@ -79,21 +79,21 @@ impl ArgAttribute {
 
     fn parse_argument(&mut self, arg: &NestedMeta) -> Result<(), Diagnostic> {
         match arg {
-            NestedMeta::Meta(Meta::Word(ident)) => {
-                if ident == "default" {
+            NestedMeta::Meta(Meta::Path(path)) => {
+                if path_eq(&path, "default") {
                     if self.default.is_some() {
-                        bail_span!(ident, "Default already set");
+                        bail_span!(path, "Default already set");
                     }
                     let expr = parse_quote!(Default::default());
                     self.default = Some(expr);
-                } else if ident == "optional" {
+                } else if path_eq(&path, "optional") {
                     self.optional = true;
                 } else {
-                    bail_span!(ident, "Unrecognised pyarg attribute");
+                    bail_span!(path, "Unrecognised pyarg attribute");
                 }
             }
             NestedMeta::Meta(Meta::NameValue(name_value)) => {
-                if name_value.ident == "default" {
+                if path_eq(&name_value.path, "default") {
                     if self.default.is_some() {
                         bail_span!(name_value, "Default already set");
                     }
@@ -107,7 +107,7 @@ impl ArgAttribute {
                         }
                         _ => bail_span!(name_value, "Expected string value for default argument"),
                     }
-                } else if name_value.ident == "optional" {
+                } else if path_eq(&name_value.path, "optional") {
                     match name_value.lit {
                         Lit::Bool(ref val) => {
                             self.optional = val.value;
@@ -147,6 +147,13 @@ fn generate_field(field: &Field) -> Result<TokenStream2, Diagnostic> {
     };
 
     let name = &field.ident;
+    if let Some(name) = name {
+        if name.to_string().starts_with("_phantom") {
+            return Ok(quote! {
+                #name: std::marker::PhantomData,
+            });
+        }
+    }
     let middle = quote! {
         .map(|x| ::rustpython_vm::pyobject::TryFromObject::try_from_object(vm, x)).transpose()?
     };
@@ -207,8 +214,9 @@ pub fn impl_from_args(input: DeriveInput) -> Result<TokenStream2, Diagnostic> {
     };
 
     let name = input.ident;
+    let (impl_generics, ty_generics, where_clause) = input.generics.split_for_impl();
     let output = quote! {
-        impl ::rustpython_vm::function::FromArgs for #name {
+        impl #impl_generics ::rustpython_vm::function::FromArgs for #name #ty_generics #where_clause {
             fn from_args(
                 vm: &::rustpython_vm::VirtualMachine,
                 args: &mut ::rustpython_vm::function::PyFuncArgs
